@@ -1,4 +1,4 @@
-# encoding: utf-8
+# frozen_string_literal: true
 
 # prawn/core/page.rb : Implements low-level representation of a PDF page
 #
@@ -12,21 +12,48 @@ require_relative 'graphics_state'
 module PDF
   module Core
     class Page #:nodoc:
-      attr_accessor :document, :margins, :stack
+      attr_accessor :art_indents, :bleeds, :crops, :document, :margins, :stack,
+        :trims
       attr_writer :content, :dictionary
 
-      def initialize(document, options={})
+      ZERO_INDENTS = {
+        left: 0,
+        bottom: 0,
+        right: 0,
+        top: 0
+      }.freeze
+
+      def initialize(document, options = {})
         @document = document
-        @margins  = options[:margins] || { :left    => 36,
-                                           :right   => 36,
-                                           :top     => 36,
-                                           :bottom  => 36  }
+        @margins  = options[:margins] || { left: 36,
+                                           right: 36,
+                                           top: 36,
+                                           bottom: 36 }
+        @crops = options[:crops] || ZERO_INDENTS
+        @bleeds = options[:bleeds] || ZERO_INDENTS
+        @trims = options[:trims] || ZERO_INDENTS
+        @art_indents = options[:art_indents] || ZERO_INDENTS
         @stack = GraphicStateStack.new(options[:graphic_state])
-        if options[:object_id]
-          init_from_object(options)
-        else
-          init_new_page(options)
-        end
+        @size     = options[:size] || 'LETTER'
+        @layout   = options[:layout] || :portrait
+
+        @stamp_stream      = nil
+        @stamp_dictionary  = nil
+
+        @content = document.ref({})
+        content << 'q' << "\n"
+        @dictionary = document.ref(
+          Type: :Page,
+          Parent: document.state.store.pages,
+          MediaBox: dimensions,
+          CropBox: crop_box,
+          BleedBox: bleed_box,
+          TrimBox: trim_box,
+          ArtBox: art_box,
+          Contents: content
+        )
+
+        resources[:ProcSet] = %i[PDF Text ImageB ImageC ImageI]
       end
 
       def graphic_state
@@ -45,16 +72,16 @@ module PDF
       end
 
       def size
-        defined?(@size) && @size || dimensions[2,2]
+        defined?(@size) && @size || dimensions[2, 2]
       end
 
       def in_stamp_stream?
-        !!@stamp_stream
+        !@stamp_stream.nil?
       end
 
       def stamp_stream(dictionary)
-        @stamp_stream     = ""
         @stamp_dictionary = dictionary
+        @stamp_stream     = @stamp_dictionary.stream
         graphic_stack_size = stack.stack.size
 
         document.save_graphics_state
@@ -65,8 +92,6 @@ module PDF
           document.restore_graphics_state
         end
 
-        @stamp_dictionary << @stamp_stream
-
         @stamp_stream      = nil
         @stamp_dictionary  = nil
       end
@@ -76,7 +101,8 @@ module PDF
       end
 
       def dictionary
-        defined?(@stamp_dictionary) && @stamp_dictionary || document.state.store[@dictionary]
+        defined?(@stamp_dictionary) && @stamp_dictionary ||
+          document.state.store[@dictionary]
       end
 
       def resources
@@ -116,62 +142,67 @@ module PDF
           dictionary.data[:Contents].each do |stream|
             stream.stream.compress! if document.compression_enabled?
           end
-        else
-          content.stream.compress! if document.compression_enabled?
+        elsif document.compression_enabled?
+          content.stream.compress!
         end
       end
 
-      def imported_page?
-        @imported_page
-      end
-
       def dimensions
-        return inherited_dictionary_value(:MediaBox) if imported_page?
-
         coords = PDF::Core::PageGeometry::SIZES[size] || size
-        [0,0] + case(layout)
-        when :portrait
-          coords
-        when :landscape
-          coords.reverse
-        else
-          raise PDF::Core::Errors::InvalidPageLayout,
-            "Layout must be either :portrait or :landscape"
-        end
+        [0, 0] +
+          case layout
+          when :portrait
+            coords
+          when :landscape
+            coords.reverse
+          else
+            raise PDF::Core::Errors::InvalidPageLayout,
+              'Layout must be either :portrait or :landscape'
+          end
+      end
+
+      def art_box
+        left, bottom, right, top = dimensions
+        [
+          left + art_indents[:left],
+          bottom + art_indents[:bottom],
+          right - art_indents[:right],
+          top - art_indents[:top]
+        ]
+      end
+
+      def bleed_box
+        left, bottom, right, top = dimensions
+        [
+          left + bleeds[:left],
+          bottom + bleeds[:bottom],
+          right - bleeds[:right],
+          top - bleeds[:top]
+        ]
+      end
+
+      def crop_box
+        left, bottom, right, top = dimensions
+        [
+          left + crops[:left],
+          bottom + crops[:bottom],
+          right - crops[:right],
+          top - crops[:top]
+        ]
+      end
+
+      def trim_box
+        left, bottom, right, top = dimensions
+        [
+          left + trims[:left],
+          bottom + trims[:bottom],
+          right - trims[:right],
+          top - trims[:top]
+        ]
       end
 
       private
 
-      def init_from_object(options)
-        @dictionary = options[:object_id].to_i
-
-        unless dictionary.data[:Contents].is_a?(Array) # content only on leafs
-          @content    = dictionary.data[:Contents].identifier
-        end
-
-        @stamp_stream      = nil
-        @stamp_dictionary  = nil
-        @imported_page     = true
-      end
-
-      def init_new_page(options)
-        @size     = options[:size]    ||  "LETTER"
-        @layout   = options[:layout]  || :portrait
-
-        @stamp_stream      = nil
-        @stamp_dictionary  = nil
-        @imported_page     = false
-
-        @content    = document.ref({})
-        content << "q" << "\n"
-        @dictionary = document.ref(:Type        => :Page,
-                                   :Parent      => document.state.store.pages,
-                                   :MediaBox    => dimensions,
-                                   :Contents    => content)
-
-        resources[:ProcSet] = [:PDF, :Text, :ImageB, :ImageC, :ImageI]
-      end
-
       # some entries in the Page dict can be inherited from parent Pages dicts.
       #
       # Starting with the current page dict, this method will walk up the
@@ -183,12 +214,10 @@ module PDF
       def inherited_dictionary_value(key, local_dict = nil)
         local_dict ||= dictionary.data
 
-        if local_dict.has_key?(key)
+        if local_dict.key?(key)
           local_dict[key]
-        elsif local_dict.has_key?(:Parent)
+        elsif local_dict.key?(:Parent)
           inherited_dictionary_value(key, local_dict[:Parent].data)
-        else
-          nil
         end
       end
     end
