@@ -1,4 +1,4 @@
-# encoding: utf-8
+# frozen_string_literal: true
 
 # prawn/core/text.rb : Implements low level text helpers for Prawn
 #
@@ -11,20 +11,26 @@ module PDF
     module Text #:nodoc:
       # These should be used as a base. Extensions may build on this list
       #
-      VALID_OPTIONS = [:kerning, :size, :style]
-      MODES = { :fill => 0, :stroke => 1, :fill_stroke => 2, :invisible => 3,
-                :fill_clip => 4, :stroke_clip => 5, :fill_stroke_clip => 6,
-                :clip => 7 }
+      VALID_OPTIONS = %i[kerning size style].freeze
+      MODES = {
+        fill: 0,
+        stroke: 1,
+        fill_stroke: 2,
+        invisible: 3,
+        fill_clip: 4, stroke_clip: 5,
+        fill_stroke_clip: 6,
+        clip: 7
+      }.freeze
 
       attr_reader :skip_encoding
 
-      # Low level call to set the current font style and extract text options from
-      # an options hash. Should be called from within a save_font block
+      # Low level call to set the current font style and extract text options
+      # from an options hash. Should be called from within a save_font block
       #
       def process_text_options(options)
         if options[:style]
-          raise "Bad font family" unless font.family
-          font(font.family, :style => options[:style])
+          raise 'Bad font family' unless font.family
+          font(font.family, style: options[:style])
         end
 
         # must compare against false to keep kerning on as default
@@ -40,23 +46,23 @@ module PDF
       # Defaults to true
       #
       def default_kerning?
-        return true if !defined?(@default_kerning)
+        return true unless defined?(@default_kerning)
         @default_kerning
       end
 
-      # Call with a boolean to set the document-wide kerning setting. This can be
-      # overridden using the :kerning text option when drawing text or a text
+      # Call with a boolean to set the document-wide kerning setting. This can
+      # be overridden using the :kerning text option when drawing text or a text
       # box.
       #
       #   pdf.default_kerning = false
-      #   pdf.text("hello world")                   # text is not kerned
-      #   pdf.text("hello world", :kerning => true) # text is kerned
+      #   pdf.text('hello world')                   # text is not kerned
+      #   pdf.text('hello world', :kerning => true) # text is kerned
       #
       def default_kerning(boolean)
         @default_kerning = boolean
       end
 
-      alias_method :default_kerning=, :default_kerning
+      alias default_kerning= default_kerning
 
       # Call with no argument to retrieve the current default leading.
       #
@@ -65,12 +71,12 @@ module PDF
       # box.
       #
       #   pdf.default_leading = 7
-      #   pdf.text("hello world")                # a leading of 7 is used
-      #   pdf.text("hello world", :leading => 0) # a leading of 0 is used
+      #   pdf.text('hello world')                # a leading of 7 is used
+      #   pdf.text('hello world', :leading => 0) # a leading of 0 is used
       #
       # Defaults to 0
       #
-      def default_leading(number=nil)
+      def default_leading(number = nil)
         if number.nil?
           defined?(@default_leading) && @default_leading || 0
         else
@@ -78,7 +84,7 @@ module PDF
         end
       end
 
-      alias_method :default_leading=, :default_leading
+      alias default_leading= default_leading
 
       # Call with no argument to retrieve the current text direction.
       #
@@ -87,8 +93,8 @@ module PDF
       # box.
       #
       #   pdf.text_direction = :rtl
-      #   pdf.text("hello world")                     # prints "dlrow olleh"
-      #   pdf.text("hello world", :direction => :ltr) # prints "hello world"
+      #   pdf.text('hello world')                     # prints 'dlrow olleh'
+      #   pdf.text('hello world', :direction => :ltr) # prints 'hello world'
       #
       # Valid directions are:
       #
@@ -100,7 +106,7 @@ module PDF
       # * When printing left-to-right, the default text alignment is :left
       # * When printing right-to-left, the default text alignment is :right
       #
-      def text_direction(direction=nil)
+      def text_direction(direction = nil)
         if direction.nil?
           defined?(@text_direction) && @text_direction || :ltr
         else
@@ -108,7 +114,7 @@ module PDF
         end
       end
 
-      alias_method :text_direction=, :text_direction
+      alias text_direction= text_direction
 
       # Call with no argument to retrieve the current fallback fonts.
       #
@@ -121,16 +127,16 @@ module PDF
       # Call with an empty array to turn off fallback fonts
       #
       # file = "#{Prawn::DATADIR}/fonts/gkai00mp.ttf"
-      # font_families["Kai"] = {
-      #   :normal => { :file => file, :font => "Kai" }
+      # font_families['Kai'] = {
+      #   :normal => { :file => file, :font => 'Kai' }
       # }
       # file = "#{Prawn::DATADIR}/fonts/Action Man.dfont"
-      # font_families["Action Man"] = {
-      #   :normal      => { :file => file, :font => "ActionMan" },
+      # font_families['Action Man'] = {
+      #   :normal      => { :file => file, :font => 'ActionMan' },
       # }
-      # fallback_fonts ["Times-Roman", "Kai"]
-      # font "Action Man"
-      # text "hello ƒ 你好"
+      # fallback_fonts ['Times-Roman', 'Kai']
+      # font 'Action Man'
+      # text 'hello ƒ 你好'
       # > hello prints in Action Man
       # > ƒ prints in Times-Roman
       # > 你好 prints in Kai
@@ -142,7 +148,7 @@ module PDF
       # * Increased overhead when fallback fonts are declared as each glyph is
       #   checked to see whether it exists in the current font
       #
-      def fallback_fonts(fallback_fonts=nil)
+      def fallback_fonts(fallback_fonts = nil)
         if fallback_fonts.nil?
           defined?(@fallback_fonts) && @fallback_fonts || []
         else
@@ -150,7 +156,7 @@ module PDF
         end
       end
 
-      alias_method :fallback_fonts=, :fallback_fonts
+      alias fallback_fonts= fallback_fonts
 
       # Call with no argument to retrieve the current text rendering mode.
       #
@@ -158,7 +164,7 @@ module PDF
       # text rendering mode.
       #
       #   pdf.text_rendering_mode(:stroke) do
-      #     pdf.text("Outlined Text")
+      #     pdf.text('Outlined Text')
       #   end
       #
       # Valid modes are:
@@ -169,14 +175,18 @@ module PDF
       # * :invisible        - invisible text
       # * :fill_clip        - fill text then add to path for clipping
       # * :stroke_clip      - stroke text then add to path for clipping
-      # * :fill_stroke_clip - fill then stroke text, then add to path for clipping
+      # * :fill_stroke_clip - fill then stroke text, then add to path for
+      #                       clipping
       # * :clip             - add text to path for clipping
-      def text_rendering_mode(mode=nil)
-        return (defined?(@text_rendering_mode) && @text_rendering_mode || :fill) if mode.nil?
-        unless MODES.key?(mode)
-          raise ArgumentError, "mode must be between one of #{MODES.keys.join(', ')} (#{mode})"
+      def text_rendering_mode(mode = nil)
+        if mode.nil?
+          return defined?(@text_rendering_mode) && @text_rendering_mode || :fill
         end
-        original_mode = self.text_rendering_mode
+        unless MODES.key?(mode)
+          raise ArgumentError,
+            "mode must be between one of #{MODES.keys.join(', ')} (#{mode})"
+        end
+        original_mode = text_rendering_mode
 
         if original_mode == mode
           yield
@@ -197,8 +207,10 @@ module PDF
       # For horizontal text, a positive value will increase the space.
       # For veritical text, a positive value will decrease the space.
       #
-      def character_spacing(amount=nil)
-        return defined?(@character_spacing) && @character_spacing || 0 if amount.nil?
+      def character_spacing(amount = nil)
+        if amount.nil?
+          return defined?(@character_spacing) && @character_spacing || 0
+        end
         original_character_spacing = character_spacing
         if original_character_spacing == amount
           yield
@@ -215,7 +227,7 @@ module PDF
       # For horizontal text, a positive value will increase the space.
       # For veritical text, a positive value will decrease the space.
       #
-      def word_spacing(amount=nil)
+      def word_spacing(amount = nil)
         return defined?(@word_spacing) && @word_spacing || 0 if amount.nil?
         original_word_spacing = word_spacing
         if original_word_spacing == amount
@@ -230,29 +242,61 @@ module PDF
         end
       end
 
+      # Set the horizontal scaling. amount is a number specifying the
+      # percentage of the normal width.
+      def horizontal_text_scaling(amount = nil)
+        if amount.nil?
+          return defined?(@horizontal_text_scaling) &&
+                 @horizontal_text_scaling || 100
+        end
+
+        original_horizontal_text_scaling = horizontal_text_scaling
+        if original_horizontal_text_scaling == amount
+          yield
+        else
+          @horizontal_text_scaling = amount
+          add_content "\n#{PDF::Core.real(amount)} Tz"
+          yield
+          add_content "\n#{PDF::Core.real(original_horizontal_text_scaling)} Tz"
+          @horizontal_text_scaling = original_horizontal_text_scaling
+        end
+      end
+
+      # rubocop: disable Naming/UncommunicativeMethodParamName
       def add_text_content(text, x, y, options)
-        chunks = font.encode_text(text,options)
+        chunks = font.encode_text(text, options)
 
         add_content "\nBT"
 
         if options[:rotate]
           rad = options[:rotate].to_f * Math::PI / 180
-          array = [ Math.cos(rad), Math.sin(rad), -Math.sin(rad), Math.cos(rad), x, y ]
+          array = [
+            Math.cos(rad),
+            Math.sin(rad),
+            -Math.sin(rad),
+            Math.cos(rad),
+            x, y
+          ]
           add_content "#{PDF::Core.real_params(array)} Tm"
         else
-          add_content "#{PDF::Core.real_params([x,y])} Td"
+          add_content "#{PDF::Core.real_params([x, y])} Td"
         end
 
         chunks.each do |(subset, string)|
           font.add_to_current_page(subset)
-          add_content "/#{font.identifier_for(subset)} #{font_size} Tf"
+          add_content [
+            PDF::Core.pdf_object(font.identifier_for(subset), true),
+            PDF::Core.pdf_object(font_size, true),
+            'Tf'
+          ].join(' ')
 
-          operation = options[:kerning] && string.is_a?(Array) ? "TJ" : "Tj"
-          add_content PDF::Core::PdfObject(string, true) << " " << operation
+          operation = options[:kerning] && string.is_a?(Array) ? 'TJ' : 'Tj'
+          add_content PDF::Core.pdf_object(string, true) + ' ' + operation
         end
 
         add_content "ET\n"
       end
+      # rubocop: enable Naming/UncommunicativeMethodParamName
     end
   end
 end
