@@ -1,24 +1,23 @@
-# encoding: utf-8
+# frozen_string_literal: true
 
 puts "PDF::Core specs: Running on Ruby Version: #{RUBY_VERSION}"
 
-require "bundler"
+require 'bundler'
 Bundler.setup
 
-if ENV["COVERAGE"]
-  require "simplecov"
-  SimpleCov.start 
+if ENV['COVERAGE']
+  require 'simplecov'
+  SimpleCov.start
 end
 
-require_relative "../lib/pdf/core"
+require_relative '../lib/pdf/core'
 
-
-require "rspec"
-require "pdf/reader"
-require "pdf/inspector"
+require 'rspec'
+require 'pdf/reader'
+require 'pdf/inspector'
 
 RSpec.configure do |config|
-  config.treat_symbols_as_metadata_keys_with_true_values = true
+  config.disable_monkey_patching!
 end
 
 RSpec::Matchers.define :have_parseable_xobjects do
